<?php
/**
 * @file
 *      Drush integration for the Archport module.
 */

/**
 * Include Archport .inc files
 */
$dirname = dirname(__FILE__);
$includes = file_scan_directory($dirname, '/.inc$/');
foreach ($includes as $include) {
  require_once $include->filename;
}

/**
 * Define Archport's Drush Commands.
 *
 * @return array
 */
function archport_drush_command(): array {
  $items['archport-initialize'] = [
    'callback' => 'drush_archport_initialize',
    'required-arguments' => TRUE,
    'description' => t(
      'Initializes the Archport directory, settings.json, and map.json files.'
    ),
    'arguments' => [
      'archport_directory' => t(
        'The Archport Directory uri. It is recommended to use private://archport.'
      ),
    ],
  ];

  $items['archport-generate-map'] = [
    'callback' => 'drush_archport_generate_map',
    'required-arguments' => TRUE,
    'description' => t(
      'Generates a new map.json file based on data found in the Archport Directory.'
    ),
    'arguments' => [
      'archport_directory' => t(
        'The Archport Directory uri. It is recommended to use private://archport.'
      ),
    ],
  ];

  $items['archport-import-users'] = [
    'callback' => 'drush_archport_import_users',
    'required-arguments' => TRUE,
    'description' => t(
      'Imports users from map.json if they have not been mapped to an existing user. Updates map.json with the new target_* info.'
    ),
    'arguments' => [
      'archport_directory' => t(
        'The Archport Directory uri. It is recommended to use private://archport.'
      ),
    ],
  ];

  $items['archport-export-single-node'] = [
    'callback' => 'drush_archport_export_node',
    'required-arguments' => TRUE,
    'description' => t('Saves an node into the archport format.'),
    'arguments' => [
      'archport_directory' => t(
        'The Archport Directory uri. It is recommended to use private://archport.'
      ),
      'id' => t('The id of the node.'),
    ],
  ];

  $items['archport-import-single-node'] = [
    'callback' => 'drush_archport_import_node',
    'required-arguments' => TRUE,
    'description' => t('Imports a single node into your target site.'),
    'arguments' => [
      'archport_directory' => t(
        'The Archport Directory uri. It is recommended to use private://archport.'
      ),
      'id' => t('The id of the exported node.'),
      'type' => t('The machine name type of the node.'),
    ],
  ];

  $items['archport-export-content-type'] = [
    'callback' => 'drush_archport_export_content_type',
    'required-arguments' => TRUE,
    'description' => t(
      'Saves all nodes of a given content type into the archport format.'
    ),
    'arguments' => [
      'archport_directory' => t(
        'The Archport Directory uri. It is recommended to use private://archport.'
      ),
      'type' => t('The machine name of the content type.'),
    ],
  ];

  $items['archport-export-all-nodes'] = [
    'callback' => 'drush_archport_export_all_nodes',
    'required-arguments' => TRUE,
    'description' => t('Saves all nodes into the archport format.'),
    'arguments' => [
      'archport_directory' => t(
        'The Archport Directory uri. It is recommended to use private://archport.'
      ),
    ],
  ];

  $items['archport-import-content-type'] = [
    'callback' => 'drush_archport_import_content_type',
    'required-arguments' => TRUE,
    'description' => t(
      'Imports all nodes of a given content type from the given Archport source into the target site.'
    ),
    'arguments' => [
      'archport_directory' => t(
        'The Archport Directory uri. It is recommended to use private://archport.'
      ),
      'type' => t('The machine name of the source content type.'),
    ],
  ];

  $items['archport-import-all-nodes'] = [
    'callback' => 'drush_archport_import_all_nodes',
    'required-arguments' => TRUE,
    'description' => t(
      'Imports all nodes from the given Archport source into the target site.'
    ),
    'arguments' => [
      'archport_directory' => t(
        'The Archport Directory uri. It is recommended to use private://archport.'
      ),
    ],
  ];
  return $items;
}

/**
 * @param string $archport_directory A file uri to the archport directory. Like
 *                                   private://archport.
 *
 * @return bool
 */
function drush_archport_initialize(string $archport_directory): bool {
  $prepare_target_directory = _archport_prepare_directory($archport_directory);
  if (!$prepare_target_directory) {
    echo t(
        'Archport - initialize - Failed to prepare target directory. Aborting.'
      ) . "\n";
    return FALSE;
  }
  $write_settings = _archport_write_initial_settings(
    $archport_directory
  );
  if (!$write_settings) {
    echo t(
        'Archport - initialize - Failed to write initial settings.json. Aborting.'
      ) . "\n";
    return FALSE;
  }
  $write_map = drush_archport_generate_map($archport_directory);
  if (!$write_map) {
    echo t(
        'Archport - initialize - Failed to write initial map.json. Aborting.'
      ) . "\n";
    return FALSE;
  }
  echo t(
      'Archport - initialize - Succeeded. Please go edit your map.json and settings.json files.'
    ) . "\n";
  return TRUE;
}

/**
 * Generate map.json from all data.json files in $source. Optionally write to
 * a new file at $output.
 *
 * @param string $archport_directory A file uri to the archport directory. Like
 *                                   private://archport.
 *
 * @return bool
 */
function drush_archport_generate_map(string $archport_directory): bool {
  // Check if things have been initialized.
  if (!file_exists($archport_directory)) {
    echo t(
        'Archport - import_users - Destination directory does not exist. Please run archport-initialize before continuing.'
      ) . "\n";
    return FALSE;
  }
  $data_json_files = drush_scan_directory($archport_directory, '$data\.json$');
  foreach ($data_json_files as $file_info) {
    $data = json_decode(file_get_contents($file_info->filename), TRUE);
    _archport_write_single_node_map($archport_directory, $data);
  }
  $map = _archport_read_map($archport_directory);
  if ($map === FALSE) {
    echo t(
        'Archport - generate_map - Failed to read map array from target file. Aborting.'
      ) . "\n";
    return FALSE;
  }
  if (!isset($map['content_types'])) {
    $map['content_types'] = [];
  }
  $content_types = _archport_get_all_node_entity_types();
  foreach ($content_types as $type) {
    if (!array_key_exists($type, $map['content_types'])) {
      $map['content_types'][$type] = '';
    }
  }
  if (!isset($map['field_types'])) {
    $map['field_types'] = [];
  }
  $fields = _archport_get_all_node_field_names();
  if (!array_key_exists('field_types', $map)) {
    $map['field_types'] = [];
  }
  if (!isset($map['fields'])) {
    $map['fields'] = [];
  }
  foreach ($fields as $field) {
    if (!array_key_exists($field, $map['fields'])) {
      $map['fields'][$field] = '';
    }
    if (!array_key_exists($field, $map['field_types'])) {
      $map['field_types'][$field] = '';
    }
  }
  $write = _archport_write_map_file($archport_directory, $map);
  if (!$write) {
    echo t(
        'Archport - generate_map - Failed to write map array to target file.'
      ) . "\n";
    return FALSE;
  }
  return TRUE;
}

/**
 * Loops though users in map.json and creates them in Drupal. Updates
 * map.json with the new target_* info for the imported users.
 *
 * @param string $archport_directory A file uri to the archport directory. Like
 *                                   private://archport.
 *
 * @return bool
 */
function drush_archport_import_users($archport_directory) {
  // Check if things have been initialized.
  if (!file_exists($archport_directory) || !file_exists(
      $archport_directory . '/map.json'
    ) || !file_exists($archport_directory . '/settings.json')) {
    echo t(
        'Archport - import_users - Destination directory, map.json, and/or settings.json do not exist. Please run archport-initialize and then edit map.json and settings.json before continuing.'
      ) . "\n";
    return FALSE;
  }
  // We need the map in order to deal with how we export data.
  $map = _archport_read_map($archport_directory);
  if ($map === FALSE) {
    echo t(
        'Archport - import_users - Failed to read map.json. Please check your destination file and folder permissions. Aborting.'
      ) . "\n";
    return FALSE;
  }
  $settings = _archport_read_settings($archport_directory);
  if ($settings === FALSE) {
    echo t(
        'Archport - import_users - Failed to read settings.json. Please check your destination file and folder permissions. Aborting.'
      ) . "\n";
    return FALSE;
  }
  $import_result = _archport_import_users($settings, $map);
  if ($import_result) {
    // Let's get the new user id's and update map.json.
    foreach ($map['users'] as $key => $user) {
      if ($user['source_uid'] === 0 || $user['source_uid'] === "0") {
        // Anonymous user. Skip.
        continue;
      }
      if ($user['source_uid'] === 1 || $user['source_uid'] === "1") {
        // Superadmin user. Skip.
        continue;
      }
      if (!empty($user_data['target_uid'])) {
        // User already mapped to existing user. Skip
        continue;
      }
      $userLoad = user_load_by_name($user['source_username']);
      if (!empty($userLoad)) {
        $user_object = $userLoad;
        if (empty($map['users'][$key]['target_uid'])) {
          $map['users'][$key]['target_uid'] = $user_object->uid;
        }
        if (empty($map['users'][$key]['target_username'])) {
          $map['users'][$key]['target_username'] = $user['source_username'];
        }
        if (empty($map['users'][$key]['target_email'])) {
          $map['users'][$key]['target_email'] = $user_object->mail;
        }
        if (empty($map['users'][$key]['target_created'])) {
          $map['users'][$key]['target_created'] = $user_object->created;
        }
        if (empty($map['users'][$key]['target_updated'])) {
          $map['users'][$key]['target_updated'] = $user_object->changed;
        }
      }
    }
    _archport_write_map_file($archport_directory, $map);
  }
  return TRUE;
}


/**
 * Export a single node.
 *
 * @param string $archport_directory A file uri to the archport directory. Like
 *                                   private://archport.
 * @param int $id The node id.
 *
 * @return bool|string
 */
function drush_archport_export_node(string $archport_directory, int $id) {
  // Check if things have been initialized.
  if (!file_exists($archport_directory) || !file_exists(
      $archport_directory . '/map.json'
    ) || !file_exists($archport_directory . '/settings.json')) {
    echo t(
        'Archport - import_users - Destination directory, map.json, and/or settings.json do not exist. Please run archport-initialize and then edit map.json and settings.json before continuing.'
      ) . "\n";
    return FALSE;
  }
  echo t('Archport - export_node - Exporting node id ') . $id . t(
      ' to destination directory: '
    ) . $archport_directory . "\n";
  $current_node = node_load($id);
  if (!$current_node) {
    echo t("Archport - export_node - Failed to load node. Aborting") . "\n";
    return FALSE;
  }

  // Prepare the destination directory, no sense continuing if this
  // doesn't work.
  $export_node_to_directory = _archport_prepare_node_dir(
    $archport_directory,
    $current_node->type,
    $id
  );
  if (!$export_node_to_directory) {
    return FALSE;
  }

  // Start $node array.
  $node['nid'] = $id;
  $node['uid'] = $current_node->uid;
  $node['title'] = $current_node->title;
  $node['status'] = $current_node->status;
  $node['type'] = $current_node->type;
  $node['created'] = $current_node->created;
  $node['changed'] = $current_node->changed;
  $alias = drupal_get_path_alias('node/' . $id);
  if ($alias !== 'node/' . $id) {
    $node['alias'] = $alias;
  }

  // Load the body if it is set.
  if (isset($current_node->body)) {
    $node['body'] = $current_node->body['und'][0]['value'];
    $node['body_format'] = $current_node->body['und'][0]['format'];
    $node['body_safe'] = $current_node->body['und'][0]['safe_value'];
    // Load the summary only if it is set, and is not empty.
    if (!empty($current_node->body['und'][0]['safe_summary'])) {
      $node['body_summary'] = $current_node->body['und'][0]['summary'];
      $node['body_summary_safe'] = $current_node->body['und'][0]['safe_summary'];
    }
  }

  $user = _archport_load_user($current_node->uid);

  // Deal with fields.
  // Fields will contain any taxonomies, so initiate that.
  $taxonomies = [];
  $terms = []; // We will process these into $taxonomies.

  // Fields will contain any files, so initiate that.
  $files = [];

  // And the rest go here.
  $fields = [];

  // Find fields
  foreach ($current_node as $parameter => $value) {
    if (!empty($value) && substr_count($parameter, 'field_') > 0) {
      $field_name = $parameter;
      // Is the field a term ref?
      if (isset($value['und']) && isset($value['und'][0]) && isset($value['und'][0]['tid'])) {
        // It is a term ref.
        foreach ($value['und'] as $term) {
          $loadTerm = _archport_load_term($term['tid']);
          $loadTerm['field_name'] = $field_name;
          if ($loadTerm) {
            $terms[] = $loadTerm;
          }
        }
      }
      elseif (isset($value['und']) && isset($value['und'][0]) && isset($value['und'][0]['filename'])) {
        foreach ($value['und'] as $item) {
          $file_array = $item;
          $file_array['field_name'] = $field_name;
          $files[] = $file_array;
        }
      }
      elseif (isset($value['und'])) {
        foreach ($value['und'] as $item) {
          $field_array = $item;
          $field_array['field_name'] = $field_name;
          $fields[] = $field_array;
        }
      }
      else {
        echo t('Archport export_node - We do not know how to process field: ') . $field_name . t(' so we are skipping it.') . "\n";
      }
    }
  }

  // Now that we have found any taxonomies, parse the data into $taxonomies.
  foreach ($terms as $term) {
    $taxonomies[$term['vocabulary']['vid']]['vid'] = $term['vocabulary']['vid'];
    $taxonomies[$term['vocabulary']['vid']]['name'] = $term['vocabulary']['name'];
    $taxonomies[$term['vocabulary']['vid']]['machine_name'] = $term['vocabulary']['machine_name'];
    $taxonomies[$term['vocabulary']['vid']]['description'] = $term['vocabulary']['description'];
    $taxonomies[$term['vocabulary']['vid']]['hierarchy'] = $term['vocabulary']['hierarchy'];
    $taxonomies[$term['vocabulary']['vid']]['weight'] = $term['vocabulary']['weight'];
    $taxonomies[$term['vocabulary']['vid']]['terms'][$term['tid']]['tid'] = $term['tid'];
    $taxonomies[$term['vocabulary']['vid']]['terms'][$term['tid']]['name'] = $term['name'];
    $taxonomies[$term['vocabulary']['vid']]['terms'][$term['tid']]['description'] = $term['description'];
    $taxonomies[$term['vocabulary']['vid']]['terms'][$term['tid']]['weight'] = $term['weight'];
    $taxonomies[$term['vocabulary']['vid']]['terms'][$term['tid']]['field_name'] = $term['field_name'];
  }

  if (!empty($files)) {
    $files = _archport_process_exported_files(
      $export_node_to_directory,
      $files
    );
  }

  $data = [
    'node' => $node,
    'users' => $user,
    'taxonomies' => $taxonomies,
    'files' => $files,
    'fields' => $fields,
  ];

  $file_path = $export_node_to_directory . '/data.json';

  $save_file = file_unmanaged_save_data(
    json_encode($data, JSON_PRETTY_PRINT),
    $file_path,
    FILE_EXISTS_REPLACE
  );

  $write_map = _archport_write_single_node_map($archport_directory, $data);

  if (!$save_file) {
    echo t(
        "Archport - save-node - Failed to save node data.json file."
      ) . "\n";
  }
  else {
    echo t(
        "Archport - save-node - Succeeded in saving node data.json file. Filepath: "
      ) . $save_file . "\n";
  }
  if (!$save_file || !$write_map) {
    return FALSE;
  }
  else {
    echo t(
        'Archport - export_node - Export of node complete. Remember to set target user id\'s in map.json if you don\'t want the imported nodes to be owned by anonymous.'
      ) . "\n";
    return $save_file;
  }
}

/**
 * Import a single node.
 *
 * @param string $archport_directory A file uri to the archport directory. Like
 *                                   private://archport.
 * @param int $id The node id.
 * @param string $type The machine name of a content type.
 *
 * @return bool
 */
function drush_archport_import_node(
  string $archport_directory,
  int $id,
  string $type
): bool {
  // Check if things have been initialized.
  if (!file_exists($archport_directory) || !file_exists(
      $archport_directory . '/map.json'
    ) || !file_exists($archport_directory . '/settings.json')) {
    echo t(
        'Archport - import_users - Destination directory, map.json, and/or settings.json do not exist. Please run archport-initialize and then edit map.json and settings.json before continuing.'
      ) . "\n";
    return FALSE;
  }
  $settings = _archport_read_settings($archport_directory);
  $map = _archport_read_map($archport_directory);
  $data = _archport_read_node_data($archport_directory, $id, $type);
  if (!$data) {
    return FALSE;
  }
  $import_vocabularies = _archport_import_vocabularies($settings, $map);
  if (!$import_vocabularies) {
    echo t(
        "Archport - import_node - Vocabulary import failed, check for log messages."
      ) . "\n";
    return FALSE;
  }

  $import_terms = _archport_import_terms($settings, $map);
  if (!$import_terms) {
    echo t(
        "Archport - import_node - Term import failed, check for log messages."
      ) . "\n";
    return FALSE;
  }

  // _archport_import_node() will also deal with files and fields.
  $source_directory = $archport_directory . '/' . $type . '/' . $id;
  $import_node = _archport_import_node(
    $source_directory,
    $data,
    $settings,
    $map
  );
  if (!$import_node) {
    echo t(
        "Archport - import_node - Node import failed, check for log messages."
      ) . "\n";
    return FALSE;
  }
  return TRUE;
}


/**
 * Load all nodes of the given type and export them.
 *
 * @param string $archport_directory A file uri to the archport directory. Like
 *                                   private://archport.
 * @param string $type The machine name of a content type.
 *
 * @return bool
 */
function drush_archport_export_content_type(
  string $archport_directory,
  string $type
): bool {
  // Check if things have been initialized.
  if (!file_exists($archport_directory) || !file_exists(
      $archport_directory . '/map.json'
    ) || !file_exists($archport_directory . '/settings.json')) {
    echo t(
        'Archport - import_users - Destination directory, map.json, and/or settings.json do not exist. Please run archport-initialize and then edit map.json and settings.json before continuing.'
      ) . "\n";
    return FALSE;
  }
  $query = new EntityFieldQuery();
  $result = $query->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', $type)
    ->execute();

  if (!empty($result['node'])) {
    echo t(
        'Archport - export_content_type - Found '
      ) . count(
        $result['node']
      ) . t(' nodes of type ') . $type . t(' to export.') . "\n";
    $ids = array_keys($result['node']);
    foreach ($ids as $id) {
      drush_archport_export_node($archport_directory, $id);
    }
  }
  else {
    echo t(
        'Archport - export_content_type - Failed to load any nodes of type '
      ) . $type . t(
        ' for export.'
      ) . "\n";
    return FALSE;
  }
  return TRUE;
}

/**
 * Load all nodes and export them.
 *
 * @param string $archport_directory A file uri to the archport directory. Like
 *                                   private://archport.
 *
 * @return bool
 */
function drush_archport_export_all_nodes(string $archport_directory): bool {
  // Check if things have been initialized.
  if (!file_exists($archport_directory) || !file_exists(
      $archport_directory . '/map.json'
    ) || !file_exists($archport_directory . '/settings.json')) {
    echo t(
        'Archport - import_users - Destination directory, map.json, and/or settings.json do not exist. Please run archport-initialize and then edit map.json and settings.json before continuing.'
      ) . "\n";
    return FALSE;
  }
  $query = new EntityFieldQuery();
  $result = $query->entityCondition('entity_type', 'node')
    ->execute();
  if (!empty($result['node'])) {
    echo t(
        'Archport - export_all_nodes - Found '
      ) . count(
        $result['node']
      ) . t(
        ' nodes to export.'
      ) . "\n";
    $ids = array_keys($result['node']);
    foreach ($ids as $id) {
      drush_archport_export_node($archport_directory, $id);
    }
  }
  else {
    echo t(
        'Archport - export_all_nodes - Failed to load any nodes for export.'
      ) . "\n";
    return FALSE;
  }
  return TRUE;
}

/**
 * Import all nodes found in the content type subdirectory of the Archport
 * directory.
 *
 * @param string $archport_directory A file uri to the archport directory. Like
 *                                   private://archport.
 * @param string $type The machine name of a content type.
 *
 * @return bool
 */
function drush_archport_import_content_type(
  string $archport_directory,
  string $type
): bool {
  // Check if things have been initialized.
  if (!file_exists($archport_directory) || !file_exists(
      $archport_directory . '/map.json'
    ) || !file_exists($archport_directory . '/settings.json')) {
    echo t(
        'Archport - import_users - Destination directory, map.json, and/or settings.json do not exist. Please run archport-initialize and then edit map.json and settings.json before continuing.'
      ) . "\n";
    return FALSE;
  }
  $source_directory = $archport_directory . '/' . $type;
  if (file_exists($source_directory)) {
    $data_json_files = drush_scan_directory($source_directory, '$data\.json$');
    if (!empty($data_json_files) && is_array($data_json_files)) {
      foreach ($data_json_files as $data_json_file) {
        $uri = drupal_dirname($data_json_file->filename);
        $explode = explode('/', $uri);
        $id = $explode[count($explode) - 1];
        $import_result = drush_archport_import_node(
          $archport_directory,
          $id,
          $type
        );
        if (!$import_result) {
          echo t(
              'Archport - import_content_type - Failed to import node. Aborting. Content type and node id were: '
            ) . t(' Type: ') . $type . t(' ID: ') . $id . "\n";
          return FALSE;
        }
      }
    }
    else {
      echo t(
          'Archport - import_content_type - Failed to import content type. Something went wrong when looking for data.json files. Directory checked: '
        ) . $source_directory . "\n";
      return FALSE;
    }
  }
  else {
    echo t(
        'Archport - import_content_type - Failed to import content type. Source directory does not exist. Directory checked: '
      ) . $source_directory . "\n";
    return FALSE;
  }
  return TRUE;
}

/**
 * Import all nodes found in the Archport directory.
 *
 * @param string $archport_directory A file uri to the archport directory. Like
 *                                   private://archport.
 *
 * @return bool
 */
function drush_archport_import_all_nodes(string $archport_directory): bool {
  // Check if things have been initialized.
  if (!file_exists($archport_directory) || !file_exists(
      $archport_directory . '/map.json'
    ) || !file_exists($archport_directory . '/settings.json')) {
    echo t(
        'Archport - import_users - Destination directory, map.json, and/or settings.json do not exist. Please run archport-initialize and then edit map.json and settings.json before continuing.'
      ) . "\n";
    return FALSE;
  }
  if (file_exists($archport_directory)) {
    $data_json_files = drush_scan_directory(
      $archport_directory,
      '$data\.json$'
    );
    if (!empty($data_json_files) && is_array($data_json_files)) {
      foreach ($data_json_files as $data_json_file) {
        $uri = drupal_dirname($data_json_file->filename);
        $explode = explode('/', $uri);
        $id = $explode[count($explode) - 1];
        $type = $explode[count($explode) - 2];
        $import_result = drush_archport_import_node(
          $archport_directory,
          $id,
          $type
        );
        if (!$import_result) {
          echo t(
              'Archport - import_all_nodes - Failed to import node. Aborting. Content type and node id were: '
            ) . t(' Type: ') . $type . t(' ID: ') . $id . "\n";
          return FALSE;
        }
      }
    }
    else {
      echo t(
          'Archport - import_all_nodes - Failed to import nodes. Something went wrong when looking for data.json files. Directory checked: '
        ) . $archport_directory . "\n";
      return FALSE;
    }
  }
  else {
    echo t(
        'Archport - import_all_nodes - Failed to import nodes. Source directory does not exist. Directory checked: '
      ) . $archport_directory . "\n";
    return FALSE;
  }
  return TRUE;
}
