<?php
/**
 * @file
 *      Archport functions that deal with exporting, importing, and otherwise,
 *      files.
 */

/**
 * Process files exported from a node.
 *
 * @param string $export_node_to_directory The subdirectory the files will be
 *                                         copied into.
 * @param array  $files                    An array of files.
 *
 * @return array|false
 */
function _archport_process_exported_files(
  string $export_node_to_directory,
  array $files
) {
  // Create a directory where we can store the files.
  $file_copy_path_public = rtrim(
      $export_node_to_directory,
      '/'
    ) . '/files/public';
  $file_copy_path_private = rtrim(
      $export_node_to_directory,
      '/'
    ) . '/files/private';
  $file_copy_path_unknown = rtrim(
      $export_node_to_directory,
      '/'
    ) . '/files/unknown';
  foreach ($files as $key => $file) {
    $uri = $file['uri'];
    $scheme = file_uri_scheme($uri);
    $sub_directory = rtrim(ltrim(file_uri_target($uri), '/'), '/');
    if ($scheme === 'public') {
      $file_destination = $file_copy_path_public . '/' . $sub_directory;
    }
    elseif ($scheme === 'private') {
      $file_destination = $file_copy_path_private . '/' . $sub_directory;
    }
    else {
      $file_destination = $file_copy_path_unknown . '/' . $sub_directory;
    }

    // Make sure the correct subdirectory exists
    $real_folder = drupal_dirname($file_destination);
    if (!file_exists($real_folder)) {
      drupal_mkdir($real_folder, 0770, TRUE);
    }
    // Copy the file.
    $copy_file = file_unmanaged_copy(
      $uri,
      $file_destination,
      FILE_EXISTS_REPLACE
    );
    if (!$copy_file) {
      echo t(
          "Archport - process_files - Failed to copy file to archport destination directory."
        ) . "\n";

      return FALSE;
    }
    else {
      $copied_relative_path = trim(
        str_replace($export_node_to_directory, '', $file_destination),
        '/'
      );
      $files[$key]['archport_path'] = $copied_relative_path;
    }
  }
  return $files;
}

/**
 * Import a single file.
 *
 * @param string $source_dir The directory the imported files will be copied
 *                           from.
 * @param array  $file_data  An array of file data.
 * @param array  $settings   The settings from settings.json.
 *
 * @return false|mixed|\stdClass
 */
function _archport_import_file(
  string $source_dir,
  array $file_data,
  array $settings
) {
  // First, sanity check that the source file actually exists.
  $source_path = $source_dir . '/' . $file_data['archport_path'];
  if (file_exists($source_path)) {
    // Figure out what directory the file should be copied into.
    // Override the files old path if that's been configured in settings.json
    $public_path = '';
    $existing_action = 'rename';
    if (!empty($settings['override_file_paths']['public']['override'])) {
      $public_path = $settings['override_file_paths']['public']['path'];
      $existing_action = $settings['override_file_paths']['public']['existing_action'];
    }
    $private_path = '';
    if (!empty($settings['override_file_paths']['private']['override'])) {
      $private_path = $settings['override_file_paths']['private']['path'];
      $existing_action = $settings['override_file_paths']['private']['existing_action'];
    }
    $unknown_path = '';
    if (!empty($settings['override_file_paths']['unknown']['override'])) {
      $unknown_path = $settings['override_file_paths']['unknown']['path'];
      $existing_action = $settings['override_file_paths']['unknown']['existing_action'];
    }
    // Now pick the correct path for the file based on its existing uri.
    if (!empty($public_path) && file_uri_scheme(
        $file_data['uri']
      ) === 'public') {
      $target_directory = $public_path;
    }
    elseif (!empty($private_path) && file_uri_scheme(
        $file_data['uri']
      ) === 'private') {
      $target_directory = $private_path;
    }
    elseif (!empty($unknown_path) && file_uri_scheme(
        $file_data['uri']
      ) !== 'public' && file_uri_scheme($file_data['uri']) !== 'private') {
      $target_directory = $unknown_path;
    }
    else {
      // We aren't overriding, so just use the directory path from the
      // old uri.
      $uri_target = file_uri_target($file_data['uri']);
      $uri_scheme = file_uri_scheme($file_data['uri']);
      $explode = explode('/', $uri_target);
      $sub = '';
      for ($i = 0; $i < count($explode) - 1; $i++) {
        $sub .= '/' . $explode[$i];
      }
      $sub = rtrim(ltrim($sub, '/'), '/');
      $target_directory = $uri_scheme . '://' . $sub;
    }

    // Make sure the target directory exists.
    if (!file_exists($target_directory)) {
      $prepare = drupal_mkdir($target_directory, 0770, TRUE);
      if (!$prepare) {
        echo t(
            "Archport - archport_import_file - Failed to prepare target directory: "
          ) . $target_directory . "\n";
        return FALSE;
      }
    }

    // Set the full uri to the final destination.
    $new_file_uri = $target_directory . '/' . $file_data['filename'];
    if (file_exists($new_file_uri) && $existing_action === 'reuse') {
      // Somehow, the file already exists. So load its info from the database.
      return _archport_load_file_object_by_uri($new_file_uri);
    }
    elseif (file_exists($new_file_uri) && $existing_action === 'replace') {
      return _archport_copy_create_imported_file(
        $source_path,
        $new_file_uri,
        $target_directory,
        $file_data,
        $existing_action
      );
    }
    else {
      // Actually copy the file to its destination and make it managed.
      return _archport_copy_create_imported_file(
        $source_path,
        $new_file_uri,
        $target_directory,
        $file_data,
        $existing_action
      );
    }
  }
  else {
    echo t(
        "Archport - archport_import_file - Failed to process file. Source file did not exist. "
      ) . $source_path . "\n";
    return FALSE;
  }
}

/**
 * Load the Drupal file object based on a given uri.
 *
 * @param string $uri The files uri. Like public://path/to/foo.bar
 *
 * @return false|mixed
 */
function _archport_load_file_object_by_uri(string $uri) {
  if (!file_exists($uri)) {
    echo t(
        "Archport - load_file_object - File did not exist. Uri: "
      ) . $uri . "\n";
    return FALSE;
  }
  $efq = new EntityFieldQuery();
  $efq->entityCondition('entity_type', 'file')
    ->propertyCondition('uri', $uri);
  $result = $efq->execute();
  if (count($result) === 1) {
    // The result is a nested array...
    $resultRow = array_pop($result);
    $fileObject = array_pop($resultRow);
    $fid = $fileObject->fid;
    return file_load($fid);
  }
  elseif (count($result) > 1) {
    echo t(
        "Archport - load_file_object - Multiple files found for the same uri. You should fix this manually. Uri: "
      ) . $uri . "\n";
    return FALSE;
  }
  else {
    echo t(
        "Archport - load_file_object - No files found for this uri. You should remove any files in the following uri's directory for the node you are importing. Uri: "
      ) . $uri . "\n";
    return FALSE;
  }
}

/**
 * When importing a file, copy it to its appropriate destination, then
 * create a Drupal file object for it, and then save that to the target
 * database.
 *
 * @param string $source_path      The path, or uri, to the source file.
 * @param string $new_file_uri     The where the file is to be copied to.
 * @param string $target_directory The directory the file is to be copied into.
 * @param array  $file_data        An array of file data.
 * @param string $existing_action  What action to taken when the file already
 *                                 exists. rename or replace.
 *
 * @return false|\stdClass
 */
function _archport_copy_create_imported_file(
  string $source_path,
  string $new_file_uri,
  string $target_directory,
  array $file_data,
  string $existing_action = 'rename'
) {
  if ($existing_action === 'rename') {
    $file_exists_action = FILE_EXISTS_RENAME;
  }
  elseif ($existing_action === 'replace') {
    $file_exists_action = FILE_EXISTS_REPLACE;
  }
  else {
    echo t(
        'Archport - copy_create_imported_file - Invalid \$existing_action value set. Aborting.'
      ) . "\n";
    return FALSE;
  }
  // Make sure the correct subdirectory exists
  $real_folder = drupal_dirname($new_file_uri);
  if (!file_exists($real_folder)) {
    drupal_mkdir($real_folder, 0770, TRUE);
  }
  // Copy the file to its destination. Replace any existing files.
  $copy_file = file_unmanaged_copy(
    $source_path,
    $new_file_uri,
    $file_exists_action
  );
  if (!$copy_file) {
    echo t(
        "Archport - archport_import_file - Failed to copy source file to target destination: "
      ) . $target_directory . '/' . $file_data['filename'] . "\n";
    return FALSE;
  }
  // Create a new object for the file.
  $file = new stdClass();
  // Figure out what uid should own the file.
  if (!empty($map['users'][$file_data['uid']]['target_uid'])) {
    $target_uid = $map['users'][$file_data['uid']]['target_uid'];
  }
  else {
    $target_uid = 0;
  }
  // Set the needed properties on the new object.
  $file->uid = $target_uid;
  $file->filename = $file_data['filename'];
  $file->uri = $copy_file;
  $file->filemime = $file_data['filemime'];
  $file->status = (int) $file_data['status'];
  if (!empty($file_data['filesize'])) {
    $file->filesize = (int) $file_data['filesize'];
  }
  if (!empty($file_data['alt'])) {
    $file->alt = $file_data['alt'];
  }
  if (!empty($file_data['title'])) {
    $file->title = $file_data['title'];
  }
  if (!empty($file_data['width'])) {
    $file->width = $file_data['width'];
  }
  if (!empty($file_data['height'])) {
    $file->height = $file_data['height'];
  }
  // Now we save.
  return file_save($file);
}
