<?php
/** @file
 *      Archport functions related to the map.json file.
 */

/**
 * Write a map.json file based on a single node's exported data.
 *
 * @param string $archport_directory A file uri to the archport directory. Like
 *                                   private://archport.
 * @param array $data The data from a node's data.json file.
 *
 * @return false|mixed|string
 */
function _archport_write_single_node_map(
  string $archport_directory,
  array $data
) {
  $map = [];
  if (file_exists($archport_directory)) {
    $mapFilePath = $archport_directory . '/map.json';
    if (file_exists($mapFilePath)) {
      $map = _archport_read_map($archport_directory);
      echo t("Archport - write_single_node_map - Map file exists.") . "\n";
    }
    else {
      echo t(
          "Archport - write_single_node_map - Map file did not exist."
        ) . "\n";
    }

    // First gather the data we need to map from the exported data.
    $users = _archport_get_node_users_from_data($data);
    $content_type = _archport_get_node_content_type_from_data($data);
    $vocabularies_and_terms = _archport_get_node_vocabularies_and_terms_from_data(
      $data
    );
    $vocabularies = $vocabularies_and_terms['vocabularies'];
    $terms = $vocabularies_and_terms['terms'];
    $fields = _archport_get_node_fields_from_data($data);

    // Now we set up the mapping structure.
    $map = _archport_map_users($map, $users);
    $map = _archport_map_content_types($map, $content_type);
    $map = _archport_map_vocabularies($map, $vocabularies);
    $map = _archport_map_terms($map, $terms);
    $map = _archport_map_fields($map, $fields);

    return _archport_write_map_file($archport_directory, $map);
  }
  else {
    echo
    'Archport - write_single_node_map - The root Archport destination directory does not exist. This function should only be run after that directory is created.';
    return FALSE;
  }
}

/**
 * Read map.json into an array.
 *
 * @param string $archport_directory A file uri to the archport directory. Like
 *                                   private://archport.
 *
 * @return false|mixed
 */
function _archport_read_map(string $archport_directory) {
  $map = [];
  if (file_exists($archport_directory . '/map.json')) {
    $mapJson = file_get_contents($archport_directory . '/map.json');
    if ($mapJson != '[]') {
      $map = json_decode($mapJson, TRUE);
      if (!$map) {
        echo t(
            "Archport - read_map - The map.json file contains invalid json. Aborting."
          ) . "\n";
        return FALSE;
      }
      else {
        return $map;
      }
    }
  }
  else {
    echo t(
        "Archport - read_map - The map.json file does not exist. Create it."
      ) . "\n";
    $save = file_save_data(
      json_encode($map, TRUE),
      $archport_directory . '/map.json'
    );
    if (!$save) {
      echo t(
          "Archport - read_map - Failed to create map.json file. Aborting."
        ) . "\n";
      return FALSE;
    }// Else just move on and return the empty array.
  }
  return $map;
}

/**
 * Take one or more user arrays and map them into map.json
 *
 * @param array $map The array from map.json.
 * @param array $users An array of user data.
 *
 * @return array
 */
function _archport_map_users(array $map, array $users): array {
  if (!isset($map['users'])) {
    $map['users'] = [];
  }
  if (!empty($users)) {
    if (isset($users['uid'])) {
      $map = _archport_map_users_process_data($map, $users);
    }
    else {
      // We should have more than one user.
      foreach ($users as $user) {
        $map = _archport_map_users_process_data($map, $user);
      }
    }
  }
  return $map;
}

/**
 * Add a given use to the map's users section.
 * This function checks to make sure existing data is not overwritten.
 *
 * @param array $map The array from map.json.
 * @param array $users An array of user data.
 *
 * @return array
 */
function _archport_map_users_process_data(array $map, array $users): array {
  if (!isset($map['users'][$users['uid']]['source_created'])) {
    $map['users'][$users['uid']]['source_created'] = $users['created'];
  }
  if (!isset($map['users'][$users['uid']]['target_created'])) {
    $map['users'][$users['uid']]['target_created'] = '';
  }
  if (!isset($map['users'][$users['uid']]['source_email'])) {
    $map['users'][$users['uid']]['source_email'] = $users['email'];
  }
  if (!isset($map['users'][$users['uid']]['target_email'])) {
    $map['users'][$users['uid']]['target_email'] = '';
  }
  if (!isset($map['users'][$users['uid']]['source_uid'])) {
    $map['users'][$users['uid']]['source_uid'] = $users['uid'];
  }
  if (!isset($map['users'][$users['uid']]['target_uid'])) {
    $map['users'][$users['uid']]['target_uid'] = '';
  }
  if (!isset($map['users'][$users['uid']]['source_updated'])) {
    $map['users'][$users['uid']]['source_updated'] = $users['updated'];
  }
  if (!isset($map['users'][$users['uid']]['target_updated'])) {
    $map['users'][$users['uid']]['target_updated'] = '';
  }
  if (!isset($map['users'][$users['uid']]['source_username'])) {
    $map['users'][$users['uid']]['source_username'] = $users['username'];
  }
  if (!isset($map['users'][$users['uid']]['target_username'])) {
    $map['users'][$users['uid']]['target_username'] = '';
  }
  return $map;
}

/**
 * Add the content machine names in $content_types to the content_types
 * field in $map.
 * This function checks to make sure existing data is not overwritten.
 *
 * @param array $map The array from map.json.
 * @param mixed $content_types An array or string of content type machine names.
 *
 * @return array
 */
function _archport_map_content_types(array $map, $content_types): array {
  // Map content types as < source > => < target >.
  if (!isset($map['content_types'])) {
    $map['content_types'] = [];
  }
  if (is_array($content_types)) {
    foreach ($content_types as $content_type) {
      if (!isset($map['content_types'][$content_type])) {
        $map['content_types'][$content_type] = '';
      }
    }
  }
  else {
    if (!isset($map['content_types'][$content_types])) {
      $map['content_types'][$content_types] = '';
    }
  }
  return $map;
}

/**
 * Add the vocabularies in $vocabularies to the map in $map.
 * This function checks to make sure existing data is not overwritten.
 *
 * @param array $map The array from map.json.
 * @param array $vocabularies An array of vocabulary data.
 *
 * @return array
 */
function _archport_map_vocabularies(array $map, array $vocabularies): array {
  // Map vocabularies with a sub array.
  if (!isset($map['vocabularies'])) {
    $map['vocabularies'] = [];
  }
  foreach ($vocabularies as $vocabulary) {
    if (!isset($map['vocabularies'][$vocabulary['name']])) {
      $map['vocabularies'][$vocabulary['name']] = [
        'source_name' => $vocabulary['name'],
        'target_name' => '',
        'source_machine_name' => $vocabulary['machine_name'],
        'target_machine_name' => '',
        'source_hierarchy' => $vocabulary['hierarchy'],
        'target_hierarchy' => '',
        'source_description' => $vocabulary['description'],
        'target_description' => '',
      ];
    }
    else {
      // Only update map if the map hasn't already been set up.
      if (!isset($map['vocabularies'][$vocabulary['name']]['source_name'])) {
        $map['vocabularies'][$vocabulary['name']]['source_name'] = $vocabulary['name'];
      }
      if (!isset($map['vocabularies'][$vocabulary['name']]['target_name'])) {
        $map['vocabularies'][$vocabulary['name']]['target_name'] = '';
      }
      if (!isset($map['vocabularies'][$vocabulary['name']]['source_machine_name'])) {
        $map['vocabularies'][$vocabulary['name']]['source_machine_name'] = $vocabulary['machine_name'];
      }
      if (!isset($map['vocabularies'][$vocabulary['name']]['target_machine_name'])) {
        $map['vocabularies'][$vocabulary['name']]['target_machine_name'] = '';
      }
      if (!isset($map['vocabularies'][$vocabulary['name']]['source_hierarchy'])) {
        $map['vocabularies'][$vocabulary['name']]['source_hierarchy'] = $vocabulary['hierarchy'];
      }
      if (!isset($map['vocabularies'][$vocabulary['name']]['target_hierarchy'])) {
        $map['vocabularies'][$vocabulary['name']]['target_hierarchy'] = '';
      }
      if (!isset($map['vocabularies'][$vocabulary['name']]['source_description'])) {
        $map['vocabularies'][$vocabulary['name']]['source_description'] = $vocabulary['description'];
      }
      if (!isset($map['vocabularies'][$vocabulary['name']]['target_description'])) {
        $map['vocabularies'][$vocabulary['name']]['target_description'] = '';
      }
    }
  }
  return $map;
}

/**
 * Add the terms in $terms to the terms' field in the map $map.
 * This function checks to make sure existing data is not overwritten.
 *
 * @param array $map The array from map.json.
 * @param array $terms An array of term data.
 *
 * @return array
 */
function _archport_map_terms(array $map, array $terms): array {
  if (!isset($map['terms'])) {
    $map['terms'] = [];
  }
  // Map terms with a sub array.
  foreach ($terms as $term) {
    $new_term = [];
    // Check if the term already exists.
    $exists = [];
    $exists_key = NULL;
    foreach ($map['terms'] as $key => $value) {
      if ($value['source_name'] === $term['name'] && $value['source_vocabulary'] === $term['vocabulary']) {
        // It already exists.
        $exists = $value;
        $exists_key = $key;
        break;
      }
    }
    if (empty($exists)) {
      $new_term = [
        'source_vocabulary' => $term['vocabulary'],
        'target_vocabulary' => '',
        'source_description' => $term['description'],
        'target_description' => '',
        'source_name' => $term['name'],
        'target_name' => '',
        'source_weight' => $term['weight'],
        'target_weight' => '',
      ];
    }
    else {
      // Only update map if the map hasn't already been set up.
      if (empty($exists['source_vocabulary'])) {
        $new_term['source_vocabulary'] = $term['vocabulary'];
      }
      else {
        $new_term['source_vocabulary'] = $exists['source_vocabulary'];
      }
      if (empty($exists['target_vocabulary'])) {
        $new_term['target_vocabulary'] = '';
      }
      else {
        $new_term['target_vocabulary'] = $exists['target_vocabulary'];
      }
      if (empty($exists['source_description'])) {
        $new_term['source_description'] = $term['description'];
      }
      else {
        $new_term['source_description'] = $exists['source_description'];
      }
      if (empty($exists['target_description'])) {
        $new_term['target_description'] = '';
      }
      else {
        $new_term['target_description'] = $exists['target_description'];
      }
      if (empty($exists['source_name'])) {
        $new_term['source_name'] = $term['name'];
      }
      else {
        $new_term['source_name'] = $exists['source_name'];
      }
      if (empty($exists['target_name'])) {
        $new_term['target_name'] = '';
      }
      else {
        $new_term['target_name'] = $exists['target_name'];
      }
      if (empty($exists['source_weight'])) {
        $new_term['source_weight'] = $term['weight'];
      }
      else {
        $new_term['source_weight'] = $exists['source_weight'];
      }
      if (empty($exists['target_weight'])) {
        $new_term['target_weight'] = '';
      }
      else {
        $new_term['target_weight'] = $exists['target_weight'];
      }
    }
    if (empty($exists_key)) {
      $map['terms'][] = $new_term;
    }
    else {
      $map['terms'][$exists_key] = $new_term;
    }
  }
  return $map;
}

/**
 * Add the fields in $fields to the field list in the map $map.
 * This function checks to make sure existing data is not overwritten.
 *
 * @param array $map The array from map.json.
 * @param array $fields An array of fields.
 *
 * @return array
 */
function _archport_map_fields(array $map, array $fields): array {
  if (!isset($map['fields'])) {
    $map['fields'] = [];
  }
  // Map fields as < source field name > => < target field name >
  foreach ($fields as $field) {
    if (!isset($map['fields'][$field])) {
      $map['fields'][$field] = '';
    }
  }
  return $map;
}

/**
 * Write the data in $map as json to the path in $map_file_path.
 *
 * @param string $archport_directory A file uri to the archport directory. Like
 *                                   private://archport.
 * @param array $map The array from map.json.
 *
 * @return false|mixed|string
 */
function _archport_write_map_file(string $archport_directory, array $map) {
  $map_uri = $archport_directory . '/map.json';
  // Now we write the actual file.
  $mapJson = json_encode($map, JSON_PRETTY_PRINT);
  $save = file_unmanaged_save_data(
    $mapJson,
    $map_uri,
    FILE_EXISTS_REPLACE
  );
  if (!$save) {
    echo t("Archport - write_map_file - Failed to save map.json.") . "\n";
    return FALSE;
  }
  else {
    echo t("Archport - write_map_file - Saved map.json to: ") . $save . "\n";
    return $save;
  }
}

