<?php
/** @file Archport functions dealing specifically with nodes. */

/**
 * Read a node's data from its data.json file.
 *
 * @param string $archport_directory A file uri to the archport directory. Like
 *                                   private://archport.
 * @param int    $id                 The node id.
 * @param string $type               The content type.
 *
 * @return false|mixed
 */
function _archport_read_node_data(
  string $archport_directory,
  int $id,
  string $type
) {
  if (file_exists(
    $archport_directory . '/' . $type . '/' . $id . '/data.json'
  )) {
    return json_decode(
      file_get_contents(
        $archport_directory . '/' . $type . '/' . $id . '/data.json'
      ),
      TRUE
    );
  }
  else {
    echo t(
        "Archport - read_node_data - Could not find node data for type and id."
      ) . "\n";
  }
  return FALSE;
}

/**
 * Try to read 'users' item from $data.
 *
 * @param array $data The node data from data.json
 *
 * @return false|mixed
 */
function _archport_get_node_users_from_data(array $data) {
  if (!isset($data['users'])) {
    echo t(
        "Archport - get_node_users_from_data - \$data['users'] was not set."
      ) . "\n";
    return FALSE;
  }
  else {
    return $data['users'];
  }
}

/**
 * Try to read the node content type from $data.
 *
 * @param array $data The node data from data.json
 *
 * @return false|mixed
 */
function _archport_get_node_content_type_from_data(array $data) {
  if (isset($data['node']['type'])) {
    return $data['node']['type'];
  }
  else {
    echo t(
        "Archport - get_node_content_type_from_data - \$data['node']['type'] was not set."
      ) . "\n";
    return FALSE;
  }
}

/**
 * Find any fields in $data. Image, File, Taxonomy, or other.
 *
 * @param array $data The node data from data.json
 *
 * @return array
 */
function _archport_get_node_fields_from_data(array $data): array {
  $fields = [];
  if (!empty($data['fields'])) {
    foreach ($data['fields'] as $field) {
      $fields[] = $field['field_name'];
    }
  }
  if (!empty($data['files'])) {
    foreach ($data['files'] as $file) {
      $fields[] = $file['field_name'];
    }
  }
  if (!empty($data['taxonomies'])) {
    foreach ($data['taxonomies'] as $taxonomy) {
      foreach ($taxonomy['terms'] as $term) {
        if (!in_array($term['field_name'], $fields)) {
          $fields[] = $term['field_name'];
        }
      }
    }
  }
  return $fields;
}

/**
 * Process and import a node.
 *
 * @param string $import_dir The directory the node will be exported from.
 * @param array  $data       The node data from data.json
 * @param array  $settings   The array from settings.json
 * @param array  $map        The array from map.json
 *
 * @return bool
 */
function _archport_import_node(
  string $import_dir,
  array $data,
  array $settings,
  array $map
): bool {
  $content_type_map = $map['content_types'];
  $target_type = $content_type_map[$data['node']['type']];
  echo t(
      "Archport - import_node - Node title is: "
    ) . $data['node']['title'] . "\n";
  echo t("Archport - import_node - target_type is: ") . $target_type . "\n";
  if (!empty($map['users'][$data['node']['uid']]['target_uid'])) {
    $target_uid = $map['users'][$data['node']['uid']]['target_uid'];
  }
  else {
    $target_uid = 0;
  }
  $newNode = new stdClass();
  $newNode->is_new = TRUE;
  $newNode->type = $target_type;
  $newNode->bundle = 'node';
  $newNode->language = LANGUAGE_NONE;

  if (!empty($data['node']['body'])) {
    // There should only ever be one body value.
    // So it should be safe to use [0] here.
    $body[$newNode->language][0]['value'] = $data['node']['body'];
    $body[$newNode->language][0]['format'] = $data['node']['body_format'];
    $body[$newNode->language][0]['safe_value'] = $data['node']['body_safe'];

    if (!empty($data['node']['body_summary'])) {
      $body[$newNode->language][0]['summary'] = $data['node']['body_summary'];
      $body[$newNode->language][0]['safe_summary'] = $data['node']['body_summary_safe'];
    }
    $newNode->body = $body;
  }

  if (!empty($data['node']['status'])) {
    $newNode->status = $data['node']['status'];
  }
  if (!empty($data['node']['title'])) {
    $newNode->title = $data['node']['title'];
  }
  node_object_prepare($newNode);
  $newNode->uid = (int) $target_uid; // After node_object_prepare, otherwise it gets overwritten.
  if ($settings['keep_created']) {
    $newNode->created = (int) $data['node']['created'];
  }
  // Drupal changes the updated date no matter what.
  foreach ($data['fields'] as $source_field) {
    $target_field_name = $map['fields'][$source_field['field_name']];
    $newNode->$target_field_name[$newNode->language][] = [
      'safe_value' => $source_field['safe_value'],
      'format' => $source_field['format'],
      'value' => $source_field['value'],
    ];
  }

  // Loop over any files that we need to process.
  foreach ($data['files'] as $source_file) {
    // Add the file as a managed file in the correct destination
    $file = _archport_import_file($import_dir, $source_file, $settings);
    if (!$file) {
      // The file functions will have output helpful messages.
      return FALSE;
    }
    $file_as_array = (array) $file;
    // Check if the display property exists, add it if it does.
    if (!empty($source_file['display'])) {
      $file_as_array['display'] = $source_file['display'];
    }
    // Check if the description property exists, add it if it does.
    if (!empty($source_file['description'])) {
      $file_as_array['description'] = $source_file['description'];
    }
    // Get the target node's field name for this file.
    $target_field = $map['fields'][$source_file['field_name']];
    // Add the file to the node in the target field.
    $newNode->$target_field[$newNode->language][] = $file_as_array;
  }

  // Loop over any taxonomy data and add terms where appropriate.
  foreach ($data['taxonomies'] as $vocabulary) {
    // We import vocabularies before we try to import nodes, so this vocab
    // should exist already.
    $target_vocabulary = taxonomy_vocabulary_machine_name_load(
      $vocabulary['machine_name']
    );
    if (empty($target_vocabulary)) {
      echo t(
          "Archport - import_node - Failed to load vocabulary by machine name when importing taxonomy data: "
        ) . $vocabulary['machine_name'] . "\n";
      return FALSE;
    }
    foreach ($vocabulary['terms'] as $term_data) {
      // We import terms before we import nodes, so this term should exist already.
      // So we need to load it.
      $target_term_load = taxonomy_get_term_by_name(
        $term_data['name'],
        $target_vocabulary->machine_name
      );
      if (count($target_term_load) > 1) {
        echo t(
            "Archport - import_node - Found multiple terms with the same name in the same vocabulary. You should manually fix this. Term: "
          ) . $term_data['name'] . "\n";
      }
      // Drupal returns an array indexed by tid, so we need to use foreach
      // to get the actual term object out of the array.
      $target_term = '';
      foreach ($target_term_load as $term_item) {
        // If we have more than one result, just pick the first one.
        // Not really any other way to deal with it.
        $target_term = $term_item;
        break;
      }
      if (empty($target_term)) {
        echo t(
            "Archport - import_node - Failed to load term by name when importing taxonomy data: "
          ) . $term_data['name'] . "\n";
        return FALSE;
      }
      $target_field_name = $map['fields'][$term_data['field_name']];
      $newNode->$target_field_name[$newNode->language][] = ['tid' => $target_term->tid];
    }
  }

  try {
    node_save($newNode);
    $query = new EntityFieldQuery();
    $result = $query->entityCondition('entity_type', 'node')
      ->entityCondition('title', $data['node']['title'])
      ->entityCondition('type', $target_type)
      ->execute();
    if ($result['node']) {
      // If there are multiple nodes with the same title, the one with the
      // largest nid should be the one we just added. So we'll use it.
      $imported_node_info = array_pop($result['node']);
      if ($settings['create_aliases']) {
        _archport_import_node_alias($imported_node_info, $data);
      }
      if ($settings['create_nid_aliases']) {
        _archport_create_nid_alias($imported_node_info, $data);
      }
      $archport_directory = _archport_get_archport_directory_from_uri(
        $import_dir
      );
      if (!empty($settings['write_apache_redirects'])) {
        _archport_write_redirect_line(
          $archport_directory,
          'apache',
          $settings,
          $imported_node_info,
          $data
        );
      }
      if (!empty($settings['write_haproxy_redirects'])) {
        _archport_write_redirect_line(
          $archport_directory,
          'haproxy',
          $settings,
          $imported_node_info,
          $data
        );
      }
    }
    return TRUE;
  } catch (Exception $e) {
    echo t("Failed to save node. Exception: ") . $e->getMessage() . "\n";
    return FALSE;
  }
}

/**
 * @param mixed $imported_node_info The info from the newly imported node.
 * @param array $data               The node data from data.json
 *
 * @return bool
 */
function _archport_import_node_alias($imported_node_info, array $data): bool {
  if (empty($data['node']['alias'])) {
    echo t(
        'Archport - import_node_alias - Data did not contain an alias to import. Aborting.'
      ) . "\n";
    return FALSE;
  }

  $node_path = 'node/' . $imported_node_info->nid;
  $import_alias = rtrim(ltrim($data['node']['alias'], '/'), '/');
  $new_path = [
    'source' => $node_path,
    'alias' => $import_alias,
  ];
  path_save($new_path);
  return TRUE;
}

/**
 * @param mixed $imported_node_info The info from the newly imported node.
 * @param array $data               The node data from data.json
 *
 * @return bool
 */
function _archport_create_nid_alias($imported_node_info, array $data): bool {
  $node_path = 'node/' . $imported_node_info->nid;
  $import_alias = rtrim(ltrim('node/' . $data['node']['nid'], '/'), '/');
  $new_path = [
    'source' => $node_path,
    'alias' => $import_alias,
  ];
  path_save($new_path);
  return TRUE;
}

function _archport_write_redirect_line(
  $archport_directory,
  $redirect_type,
  $settings,
  $create,
  $data
) {
  if (empty($settings['source_site_base_url'])) {
    echo t(
        'Archport - write_redirect_line - Setting "source_site_base_url" is not set. Aborting.'
      ) . "\n";
    return FALSE;
  }
  if (empty($settings['target_site_base_url'])) {
    echo t(
        'Archport - write_redirect_line - Setting "target_site_base_url" is not set. Aborting.'
      ) . "\n";
    return FALSE;
  }
  $source_base = rtrim($settings['source_site_base_url'], '/');
  $target_base = rtrim($settings['target_site_base_url'], '/');
  $source_node_path = 'node/' . $data['node']['nid'];
  $target_node_path = 'node/' . $create->nid;
  $source_uri_node_id = $source_base . '/' . $source_node_path;
  $target_uri_node_id = $target_base . '/' . $target_node_path;
  if (!empty($data['node']['alias'])) {
    $source_alias_uri = $source_base . '/' . trim(
        $data['node']['alias'],
        '/'
      );
    $target_alias_uri = $target_base . '/' . trim(
        $data['node']['alias'],
        '/'
      );
  }
  if ($redirect_type == 'haproxy') {
    $file_out = $archport_directory . '/haproxy_redirects.txt';
    $source_haproxy_node_id_base = file_uri_target($source_uri_node_id);
    $redirect_node_id_line = "redirect location $target_uri_node_id code 301 if { base -i $source_haproxy_node_id_base }\n";
    if (!empty($data['node']['alias']) && !empty($target_alias_uri) && !empty($source_alias_uri)) {
      $source_haproxy_alias_base = file_uri_target($source_alias_uri);
      if (!empty($settings['create_aliases'])) {
        $redirect_alias_line = "redirect location $target_alias_uri code 301 if { base -i $source_haproxy_alias_base }\n";
      }
      else {
        $redirect_alias_line = "redirect location $target_uri_node_id code 301 if { base -i $source_haproxy_alias_base }\n";
      }
    }
  }
  elseif ($redirect_type == 'apache') {
    $file_out = $archport_directory . '/apache_redirects.txt';
    $redirect_node_id_line = "Redirect 301 /" . trim(
        $source_node_path,
        '/'
      ) . " $target_uri_node_id\n";
    if (!empty($data['node']['alias']) && !empty($target_alias_uri) && !empty($source_alias_uri)) {
      if (!empty($settings['create_aliases'])) {
        $redirect_alias_line = "Redirect 301 /" . $data['node']['alias'] . " $target_alias_uri\n";
      }
      else {
        $redirect_alias_line = "Redirect 301 /" . $data['node']['alias'] . " $target_uri_node_id\n";
      }
    }
  }
  else {
    echo t(
        'Archport - write_redirect_line - redirect_type is invalid. Aborting.'
      ) . "\n";
    return FALSE;
  }

  $path = drupal_realpath($file_out);
  $fh = fopen($path, 'a');
  if (empty($fh)) {
    echo t(
        'Archport - write_redirect_line - Failed to open redirect text file. Aborting.'
      ) . "\n";
    return FALSE;
  }
  $write_node_redirect_line = fwrite($fh, $redirect_node_id_line);
  if (!$write_node_redirect_line) {
    echo t(
        'Archport - write_redirect_line - Failed to write node id redirect line.'
      ) . "\n";
  }
  if (!empty($redirect_alias_line)) {
    $write_alias_redirect_line = fwrite($fh, $redirect_alias_line);
    if (!$write_alias_redirect_line) {
      echo t(
          'Archport - write_redirect_line - Failed to write alias redirect line.'
        ) . "\n";
    }
  }
  fclose($fh);
  if (empty($write_alias_redirect_line) || empty($write_node_redirect_line)) {
    return FALSE;
  }
  else {
    return TRUE;
  }
}
