<?php
/** @file Archport functions dealing with taxonomy vocabularies and terms. */

/**
 * Load a term by its tid and turn it into a custom array.
 *
 * @param int $tid A term id.
 *
 * @return array
 */
function _archport_load_term(int $tid): array {
  $termLoad = taxonomy_term_load($tid);
  $vocabularyLoad = taxonomy_vocabulary_load($termLoad->vid);
  return [
    'tid' => $tid,
    'name' => $termLoad->name,
    'description' => $termLoad->description,
    'weight' => $termLoad->weight,
    'vocabulary' => [
      'vid' => $vocabularyLoad->vid,
      'name' => $vocabularyLoad->name,
      'machine_name' => $vocabularyLoad->machine_name,
      'description' => $vocabularyLoad->description,
      'hierarchy' => $vocabularyLoad->hierarchy,
      'weight' => $vocabularyLoad->weight,
    ],
  ];
}

/**
 * Find vocabularies and terms from an exported nodes' data in $data.
 * Return them as an array.
 *
 * @param array $data The node data from data.json
 *
 * @return array[]
 */
function _archport_get_node_vocabularies_and_terms_from_data(array $data
): array {
  $vocabularies = [];
  $terms = [];
  if (!empty($data['taxonomies'])) {
    foreach ($data['taxonomies'] as $vid => $vocabulary) {
      $vocabularies[$vid] = [
        'name' => $vocabulary['name'],
        'machine_name' => $vocabulary['machine_name'],
        'hierarchy' => $vocabulary['hierarchy'],
        'description' => $vocabulary['description'],
      ];
      foreach ($vocabulary['terms'] as $term) {
        $terms[$term['tid']] = [
          'vocabulary' => $vocabulary['name'],
          'description' => $term['description'],
          'field_name' => $term['field_name'],
          'name' => $term['name'],
          'tid' => $term['tid'],
          'weight' => $term['weight'],
        ];
      }
    }
  } // else just return the empty arrays.
  return ['vocabularies' => $vocabularies, 'terms' => $terms];
}

/**
 * Import vocabularies from map.json. Unless the settings say otherwise.
 *
 * @param array $settings The array from settings.json
 * @param array $map      The array from map.json
 *
 * @return bool
 */
function _archport_import_vocabularies(array $settings, array $map): bool {
  if ($settings['skip_vocabularies']) {
    return TRUE;
  }
  if (!$settings['create_vocabularies']) {
    return TRUE;
  }
  if (empty($map['vocabularies'])) {
    return TRUE;
  }
  $errors = FALSE;
  foreach ($map['vocabularies'] as $vocabulary) {
    $check_for_existing = taxonomy_vocabulary_machine_name_load(
      $vocabulary['source_machine_name']
    );
    if (!empty($check_for_existing)) {
      continue;
    }
    $newVocabulary = new stdClass();
    if (!empty($vocabulary['target_name'])) {
      $newVocabulary->name = $vocabulary['target_name'];
    }
    else {
      $newVocabulary->name = $vocabulary['source_name'];
    }
    if (!empty($vocabulary['target_machine_name'])) {
      $newVocabulary->machine_name = $vocabulary['target_machine_name'];
    }
    else {
      $newVocabulary->machine_name = $vocabulary['source_machine_name'];
    }
    if (!empty($vocabulary['target_description'])) {
      $newVocabulary->description = $vocabulary['target_description'];
    }
    else {
      $newVocabulary->description = $vocabulary['source_description'];
    }
    if (!empty($vocabulary['target_hierarchy'])) {
      $newVocabulary->hierarchy = $vocabulary['target_hierarchy'];
    }
    else {
      $newVocabulary->hierarchy = $vocabulary['source_hierarchy'];
    }
    $result = taxonomy_vocabulary_save($newVocabulary);
    if ($result != SAVED_NEW) {
      $errors = TRUE;
      echo t(
          "Archport - import_vocabularies - Failed to save new vocabulary with machine_name: "
        ) . $newVocabulary->machine_name . "\n";
    }
    else {
      echo t(
          "Archport - import_vocabularies - Saved new vocabulary with machine_name: "
        ) . $newVocabulary->machine_name . "\n";
    }
  }
  if ($errors) {
    return FALSE;
  }
  else {
    return TRUE;
  }
}

/**
 * Import terms from map.json. Unless the settings say otherwise.
 *
 * @param array $settings The array from settings.json
 * @param array $map      The array from map.json
 *
 * @return bool
 */
function _archport_import_terms(array $settings, array $map): bool {
  if ($settings['skip_terms']) {
    return TRUE;
  }
  if (!$settings['create_terms']) {
    return TRUE;
  }
  if (empty($map['terms'])) {
    return TRUE;
  }
  $errors = FALSE;
  foreach ($map['terms'] as $term) {
    $source_vocabulary = $term['source_vocabulary'];
    if (!empty($map['vocabularies'][$source_vocabulary]['target_machine_name'])) {
      $target_vocabulary_machine_name = $map['vocabularies'][$source_vocabulary]['target_machine_name'];
    }
    else {
      $target_vocabulary_machine_name = $map['vocabularies'][$source_vocabulary]['source_machine_name'];
    }
    $check_existing = taxonomy_get_term_by_name(
      $term['source_name'],
      $target_vocabulary_machine_name
    );
    if (!empty($check_existing)) {
      continue;
    }
    $load_vocabulary = taxonomy_vocabulary_machine_name_load(
      $target_vocabulary_machine_name
    );
    $target_vid = $load_vocabulary->vid;
    $newTerm = new stdClass();
    if (!empty($term['target_name'])) {
      $newTerm->name = $term['target_name'];
    }
    else {
      $newTerm->name = $term['source_name'];
    }
    if (!empty($term['target_description'])) {
      $newTerm->description = $term['target_description'];
    }
    else {
      $newTerm->description = $term['source_description'];
    }
    if (!empty($term['target_weight'])) {
      $newTerm->weight = $term['target_weight'];
    }
    else {
      $newTerm->weight = $term['source_weight'];
    }
    $newTerm->vocabulary_machine_name = $target_vocabulary_machine_name;
    $newTerm->vid = $target_vid;
    $result = taxonomy_term_save($newTerm);
    if ($result != SAVED_NEW) {
      $errors = TRUE;
      echo t(
          "Archport - import_terms - Failed to save new term with name: "
        ) . $newTerm->name . t(
          'In vocabulary: '
        ) . $newTerm->vocabulary_machine_name . "\n";
    }
    else {
      echo t(
          "Archport - import_terms - Saved new term with name: "
        ) . $newTerm->name . t(
          'In vocabulary: '
        ) . $newTerm->vocabulary_machine_name . "\n";
    }
  }
  if ($errors) {
    return FALSE;
  }
  else {
    return TRUE;
  }
}
