<?php
/** @file Archport functions dealing with users. */

/**
 * Load a user by their uid, then return a custom array with the loaded data.
 *
 * @param int $uid A user uid.
 *
 * @return array|false
 */
function _archport_load_user(int $uid) {
  $user = user_load($uid);
  if (!$user) {
    echo t("Archport - load_user - Failed to load user object") . "\n";
    return FALSE;
  }
  return [
    'uid' => $user->uid,
    'username' => $user->name,
    'email' => $user->mail,
    'created' => $user->created,
    'updated' => $user->changed,
  ];
}

/**
 * Create a new user based on the data found in map.json.
 *
 * @param array $mappedUser An array of user info from map.json
 *
 * @return false|mixed|\stdClass
 */
function _archport_create_user(array $mappedUser) {
  $newAccount = new stdClass();
  // TODO: Use source fields if target is blank.
  $newAccount->name = $mappedUser['target_username'];
  $newAccount->created = $mappedUser['target_created'];
  $newAccount->mail = $mappedUser['target_email'];
  $newAccount->changed = $mappedUser['target_changed'];
  $newAccount->is_new = TRUE;
  try {
    return user_save($newAccount);
  } catch (Exception $e) {
    echo t(
        "Archport - create_user - Exception thrown on user save. Exception: "
      ) . $e->getMessage() . "\n";
    return FALSE;
  }
}

/**
 * If the settings say to do so, loop through users in map.json data and
 * import them.
 *
 * @param array $settings The array from settings.json
 * @param array $map      The array from map.json
 *
 * @return bool
 */
function _archport_import_users(array $settings, array $map): bool {
  if (!$settings['create_users']) {
    // Just skip since creating users is false.
    return TRUE;
  }
  if (empty($map['users'])) {
    return TRUE;
  }

  foreach ($map['users'] as $user_data) {
    if ($user_data['source_uid'] === 0 || $user_data['source_uid'] === "0") {
      // Anonymous user. Skip.
      continue;
    }
    if ($user_data['source_uid'] === 1 || $user_data['source_uid'] === "1") {
      // Superadmin user. Skip.
      continue;
    }
    if (!empty($user_data['target_uid'])) {
      // User already mapped to existing user. Skip
      continue;
    }
    $new_user_data = new stdClass();
    if ($settings['keep_created']) {
      if (!empty($user_data['target_created'])) {
        $new_user_data->created = $user_data['target_created'];
      }
      elseif (!empty($user_data['source_created'])) {
        $new_user_data->created = $user_data['source_created'];
      }// Else skip it and let Drupal set it.
    }
    if ($settings['keep_updated']) {
      // This probably won't actually work since Drupal will override it...
      if (!empty($user_data['target_updated'])) {
        $new_user_data->changed = $user_data['target_updated'];
      }
      elseif (!empty($user_data['source_created'])) {
        $new_user_data->changed = $user_data['source_updated'];
      }// Else skip it and let Drupal set it.
    }
    if (!empty($user_data['target_email'])) {
      $new_user_data->mail = $user_data['target_email'];
    }
    elseif (!empty($user_data['source_email'])) {
      $new_user_data->mail = $user_data['source_email'];
    }// Else skip it and let Drupal set it.

    if (!empty($user_data['target_username'])) {
      $new_user_data->name = $user_data['target_username'];
    }
    elseif (!empty($user_data['source_username'])) {
      $new_user_data->name = $user_data['source_username'];
    }// Else skip it and let Drupal set it.

    // Does the user already exist?
    $userLoad = user_load_by_name($new_user_data->name);
    if (empty($userLoad)) {
      echo t(
          'Archport - import_users - Try to import user with username: '
        ) . $new_user_data->name . "\n";
      $new_user_data->is_new = TRUE;
      try {
        $saved_user = user_save($new_user_data);
        echo t(
            'Archport - import_users - Succeeded in importing user with username: '
          ) . $new_user_data->name . "\n";
      } catch (Exception $e) {
        echo t(
            'Archport - import_users - Failed to save new user. Aborting. Exception: '
          ) . $e->getMessage() . "\n";
        return FALSE;
      }
    }// Else the user exists, so just move on to the next one.
  }
  return TRUE;
}
